package net.windward.xmlreport;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

/**
 * This implements a system to import files requested via the import tag. This class must be instantiated because
 * there is no static methods in an interface. But this is a singleton, a single object will be instantiated for
 * the engine to handle all import requests and therefore must be thread safe.
 *
 * @version 5.0 19 August 2007
 */
public class SampleImportFile implements IImportFile {

	/**
	 * Read in an imported file.
	 * @param filename The filename, url, etc.
	 * @param attributes The tag properties.
	 * @param baseDirectory The base directory for reletive filenames. null if there is no base directory.
	 * @return The file contents. null if the file cannot be found.
	 * @throws java.io.IOException hit a problem reading the file.
	 */
	public LoadStream readImport(String filename, Map attributes, String baseDirectory) throws IOException {

		boolean tryAll = ! attributes.containsKey("source");

		// try a filename first
		String lcName = filename.toLowerCase();
		if ((! lcName.startsWith("http:")) && (! lcName.startsWith("ftp:")))
			if (tryAll || getAttribute(attributes, "source").equals("FILE")) {
				File file = new File(filename);
				if (file.exists())
					return new LoadStream(new FileInputStream(file), file.getAbsolutePath());
			}

		// try a URL second
		if (tryAll || getAttribute(attributes, "source").equals("URL")) {
			try {
				URL url = new URL(filename);
				return new LoadStream(url.openStream(), url.toExternalForm());
			} catch (MalformedURLException mue) {
				// nothing
			}
		}

		// try a relative filename last
		if ((tryAll || getAttribute(attributes, "source").equals("FILE"))) {
			File file = new File(filename);
			if ((baseDirectory != null) && (!file.isAbsolute())) {
				file = new File(baseDirectory, filename);
				return new LoadStream(new FileInputStream(file), file.getAbsolutePath());
			}
		}

		return null;
	}

	private static String getAttribute(Map attributes, String key) {
		if (!attributes.containsKey(key))
			return "";
		return (String) attributes.get(key);
	}
}
